from Products.Five.browser import BrowserView

from plone import api

from zope.interface.interfaces import ComponentLookupError

from zopyx.usersascontent.interfaces import IUsersAsContentSettings
from collective.relationhelpers import api as relapi


class Dashboard(BrowserView):
    """Dashboard browser view"""

    def goto_my_dashboard(self):
        """Redirect to PloneUser object of currently logged in user"""

        user = api.user.get_current()
        user_id = user.getId()
        portal = api.portal.get()

        user_folder_id = api.portal.get_registry_record(
            "user_folder_id", IUsersAsContentSettings
        )
        if user_folder_id not in portal.objectIds():
            api.portal.show_message("No user object found for you ", self.request)
            self.request.response.redirect(portal.absolute_url())
            return

        user_folder = portal[user_folder_id]

        if user_id in user_folder:
            api.portal.show_message("User object found for you ", self.request)
            user_object_id = f"{portal.absolute_url()}/{user_folder_id}/{user_id}"
            self.request.response.redirect(user_object_id)
        else:
            api.portal.show_message("No user object found for you ", self.request)
            self.request.response.redirect(portal.absolute_url())

    def my_content(self):
        catalog = api.portal.get_tool("portal_catalog")
        return catalog(Creator=api.user.get_current().getId())

    def my_forward_references(self):
        return relapi.relations(self.context, as_dict=True)

    def my_backward_references(self):
        return relapi.backrelations(self.context, as_dict=True)



# -*- coding: utf-8 -*-

################################################################
# zopyx.userascontent
# (C) 2022,  Andreas Jung, www.zopyx.com, Tuebingen, Germany
################################################################

import json

from plone.app.registry.browser import controlpanel
from plone.registry.interfaces import IRegistry
from zopyx.usersascontent import _
from zopyx.usersascontent.interfaces import IUsersAsContentSettings
from zope.component import getUtility


class UsersAsContentEditForm(controlpanel.RegistryEditForm):

    schema = IUsersAsContentSettings
    label = _(u"Users as Content settings")
    description = _(u"")

    def updateFields(self):
        super(UsersAsContentEditForm, self).updateFields()

    def updateWidgets(self):
        super(UsersAsContentEditForm, self).updateWidgets()


class UsersAsContentControlPanel(controlpanel.ControlPanelFormWrapper):
    form = UsersAsContentEditForm

    @property
    def settings(self):
        """Returns setting as dict"""
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IUsersAsContentSettings)
        result = dict()
        for name in settings.__schema__:
            result[name] = getattr(settings, name)
        return result

    def settings_json(self):
        """Returns setting as JSON"""
        return json.dumps(self.settings)



# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PLONE_FIXTURE,
    PloneSandboxLayer,
)
from plone.testing import z2

import zopyx.usersascontent


class ZopyxUsersascontentLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.app.dexterity

        self.loadZCML(package=plone.app.dexterity)
        import plone.restapi

        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=zopyx.usersascontent)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "zopyx.usersascontent:default")


ZOPYX_USERSASCONTENT_FIXTURE = ZopyxUsersascontentLayer()


ZOPYX_USERSASCONTENT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(ZOPYX_USERSASCONTENT_FIXTURE,),
    name="ZopyxUsersascontentLayer:IntegrationTesting",
)


ZOPYX_USERSASCONTENT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(ZOPYX_USERSASCONTENT_FIXTURE,),
    name="ZopyxUsersascontentLayer:FunctionalTesting",
)


ZOPYX_USERSASCONTENT_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        ZOPYX_USERSASCONTENT_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name="ZopyxUsersascontentLayer:AcceptanceTesting",
)



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""

from zopyx.usersascontent import _
from zope import schema
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class IZopyxUsersascontentLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IUsersAsContentSettings(Interface):
    """Connector settings"""

    #    enabled = schema.Bool(title=_('Users as Content enabled'),
    #                          description=None,
    #                          default=True,
    #                          required=False)

    user_folder_id = schema.TextLine(
        title=_("ID of user folder"), description=None, default="users", required=True
    )

    redirect_after_registration = schema.Bool(
        title=_("Redirect to user object after registration (first login)"),
        description=None,
        default=True,
        required=False,
    )

    redirect_always = schema.Bool(
        title=_("Redirect to user object always after login"),
        description=None,
        default=True,
        required=False,
    )



# -*- coding: utf-8 -*-
from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.testing import setRoles, TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject, queryUtility
from zopyx.usersascontent.content.plone_user import IPloneUser  # NOQA E501
from zopyx.usersascontent.testing import (
    ZOPYX_USERSASCONTENT_INTEGRATION_TESTING,  # noqa,
)

import unittest


class PloneUserIntegrationTest(unittest.TestCase):

    layer = ZOPYX_USERSASCONTENT_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.parent = self.portal

    def test_ct_plone_user_schema(self):
        fti = queryUtility(IDexterityFTI, name="PloneUser")
        schema = fti.lookupSchema()
        self.assertEqual(IPloneUser, schema)

    def test_ct_plone_user_fti(self):
        fti = queryUtility(IDexterityFTI, name="PloneUser")
        self.assertTrue(fti)

    def test_ct_plone_user_factory(self):
        fti = queryUtility(IDexterityFTI, name="PloneUser")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IPloneUser.providedBy(obj),
            u"IPloneUser not provided by {0}!".format(
                obj,
            ),
        )

    def test_ct_plone_user_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        obj = api.content.create(
            container=self.portal,
            type="PloneUser",
            id="plone_user",
        )

        self.assertTrue(
            IPloneUser.providedBy(obj),
            u"IPloneUser not provided by {0}!".format(
                obj.id,
            ),
        )

        parent = obj.__parent__
        self.assertIn("plone_user", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=obj)
        self.assertNotIn("plone_user", parent.objectIds())

    def test_ct_plone_user_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="PloneUser")
        self.assertTrue(
            fti.global_allow, u"{0} is not globally addable!".format(fti.id)
        )

    def test_ct_plone_user_filter_content_type_true(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="PloneUser")
        portal_types = self.portal.portal_types
        parent_id = portal_types.constructContent(
            fti.id,
            self.portal,
            "plone_user_id",
            title="PloneUser container",
        )
        self.parent = self.portal[parent_id]
        with self.assertRaises(InvalidParameterError):
            api.content.create(
                container=self.parent,
                type="Document",
                title="My Content",
            )



from plone import api
from datetime import datetime
from Products.CMFPlone.interfaces import IRedirectAfterLogin
from Products.CMFPlone.utils import safe_unicode
from zope.interface import implementer
from zopyx.usersascontent.interfaces import IUsersAsContentSettings


@implementer(IRedirectAfterLogin)
class RedirectAfterLoginAdapter(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def create_and_get_user_folder(self):

        portal = api.portal.get()
        user = api.user.get_current()
        user_id = user.getId()
        user_folder_id = api.portal.get_registry_record(
            "user_folder_id", IUsersAsContentSettings
        )

        with api.env.adopt_roles(roles=["Manager"]):

            if user_folder_id not in portal.objectIds():
                uf = api.content.create(
                    type="Folder", id=user_folder_id, title="Users", container=portal
                )
                api.content.transition(uf, "publish")
            uf = portal[user_folder_id]

            if user_id not in uf.objectIds():
                api.portal.show_message(
                    "We created a new user object for you", self.request
                )
                user_obj = api.content.create(
                    type="PloneUser", id=user_id, title=user_id, container=uf
                )

        return portal[user_folder_id][user_id]   

    def __call__(self, came_from=None, is_initial_login=False):

        user_obj = self.create_and_get_user_folder()

        # first login?
        if not user_obj.first_login:
            user_obj.first_login = datetime.utcnow()
            user_obj.last_login = datetime.utcnow()

            redirect_after_registration= api.portal.get_registry_record(
                "redirect_after_registration", IUsersAsContentSettings
            )
            if redirect_after_registration:
                return user_obj.absolute_url() 

        # update login dates
        user_obj.last_login = datetime.utcnow()

        redirect_always = api.portal.get_registry_record(
            "redirect_always", IUsersAsContentSettings
        )
        if redirect_always:
            return user_obj.absolute_url()

        came_from = self.request.get("came_from")
        if came_from:
            return came_from

        return self.context.absolute_url()



# -*- coding: utf-8 -*-
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.dexterity.content import Container, Item
from plone.namedfile import field as namedfile
from plone.supermodel import model
from plone.supermodel.directives import fieldset

# from z3c.form.browser.radio import RadioFieldWidget
from zope import schema
from zope.interface import implementer
from plone.app.vocabularies.catalog import CatalogSource
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from plone.autoform.directives import read_permission
from plone.autoform.directives import write_permission

from zopyx.usersascontent import _


class IPloneUser(model.Schema):
    """Marker interface and Dexterity Python Schema for PloneUser"""

    fullname = schema.TextLine(title=_(u"Your fullname"), required=False)

    email = schema.TextLine(title=_(u"Email address"), required=False)

    organization = schema.TextLine(title=_(u"Organization"), required=False)

    text = RichText(title=_(u"Information about yourself"), required=False)

    url = schema.URI(title=_(u"Link"), required=False)

    picture = namedfile.NamedBlobImage(
        title=_(u"Your picture"),
        required=False,
    )

    read_permission(first_login="cmf.ManagePortal")
    write_permission(first_login="cmf.ManagePortal")
    fieldset("User-specific dates", fields=["first_login", "last_login"])
    first_login = schema.Datetime(
        title=_(u"First login"),
        required=False,
    )

    read_permission(last_login="cmf.ManagePortal")
    write_permission(last_login="cmf.ManagePortal")
    last_login = schema.Datetime(
        title=_(u"Last login"),
        required=False,
    )

#    evil_mastermind = RelationChoice(
#        title="The Evil Mastermind",
#        vocabulary="plone.app.vocabularies.Catalog",
#        required=False,
#    )


@implementer(IPloneUser)
class PloneUser(Item):
    """Content-type class for IPloneUser"""
